"""Module contains all the supported assets that we can pass to the scan command."""
from ostorlab.cli.scan.run.assets import android_apk
from ostorlab.cli.scan.run.assets import android_aab
from ostorlab.cli.scan.run.assets import ios_ipa
from ostorlab.cli.scan.run.assets import file
from ostorlab.cli.scan.run.assets import ip
