# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: web.proto

import sys

_b = sys.version_info[0] < 3 and (lambda x: x) or (lambda x: x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()

DESCRIPTOR = _descriptor.FileDescriptor(
    name='web.proto',
    package='',
    serialized_pb=_b(
        '\n\tweb.proto\"E\n\x03web\x12\x0f\n\x07scan_id\x18\x01 \x02(\x05\x12\x16\n\x04urls\x18\x02 \x03(\x0b\x32\x08.web.url\x1a\x15\n\x03url\x12\x0e\n\x06\x64omain\x18\x01 \x02(\t')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_WEB_URL = _descriptor.Descriptor(
    name='url',
    full_name='web.url',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='domain', full_name='web.url.domain', index=0,
            number=1, type=9, cpp_type=9, label=2,
            has_default_value=False, default_value=_b("").decode('utf-8'),
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=61,
    serialized_end=82,
)

_WEB = _descriptor.Descriptor(
    name='web',
    full_name='web',
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    fields=[
        _descriptor.FieldDescriptor(
            name='scan_id', full_name='web.scan_id', index=0,
            number=1, type=5, cpp_type=1, label=2,
            has_default_value=False, default_value=0,
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
        _descriptor.FieldDescriptor(
            name='urls', full_name='web.urls', index=1,
            number=2, type=11, cpp_type=10, label=3,
            has_default_value=False, default_value=[],
            message_type=None, enum_type=None, containing_type=None,
            is_extension=False, extension_scope=None,
            options=None),
    ],
    extensions=[
    ],
    nested_types=[_WEB_URL, ],
    enum_types=[
    ],
    options=None,
    is_extendable=False,
    extension_ranges=[],
    oneofs=[
    ],
    serialized_start=13,
    serialized_end=82,
)

_WEB_URL.containing_type = _WEB
_WEB.fields_by_name['urls'].message_type = _WEB_URL
DESCRIPTOR.message_types_by_name['web'] = _WEB

web = _reflection.GeneratedProtocolMessageType('web', (_message.Message,), dict(

    url=_reflection.GeneratedProtocolMessageType('url', (_message.Message,), dict(
        DESCRIPTOR=_WEB_URL,
        __module__='web_pb2'
        # @@protoc_insertion_point(class_scope:web.url)
    ))
    ,
    DESCRIPTOR=_WEB,
    __module__='web_pb2'
    # @@protoc_insertion_point(class_scope:web)
))
_sym_db.RegisterMessage(web)
_sym_db.RegisterMessage(web.url)

# @@protoc_insertion_point(module_scope)
